@@ -19,6 +19,8 @@ struct sockaddr *str_to_sockaddr(char *,unsigned long);
 struct sockaddr *make_sockaddr(unsigned int port, char *host,
   unsigned long defaulthost);
 char *term_strherror(int err);
+int convert_to_8(int,int,un_char *,un_char *);
+int convert_from_8(int,int,un_char *,un_char *);
 
 #ifdef X_DEBUG
 static int x_debug = 0;
@@ -36,7 +38,7 @@ extern do_stats(un_char *, int, struct Client *);
 
 /*-----------------------------------------------------------------------*/
 /* Local function prototypes */
-int get_data(un_char *, int);
+int get_data(un_char *, int *, int);
 void put_data(un_char *, int);
 int get_client_data(struct Client *);
 void put_client_data(struct Client *, int);
@@ -67,13 +69,12 @@ static char *hostent_to_str(struct hostent *addr) {
   return hostname;
 }
 
-
 /* To complicate matters, I require the first client then transmit version */
 /* # and later go back and transmit the first real packet */
 /*-----------------------------------------------------------------------*/
 void do_link_out(void) {
-  int len;
-  static int sent_version = 0, old_len = 0;
+  int len, type;
+  static int sent_version = 0, old_len = 0, old_type = -1;
 
   /* Add another packet to the out packet list */
   
@@ -94,15 +95,17 @@ void do_link_out(void) {
   new_packet = 1;
 
   if (!old_len){
-    if (remote_term_version >= 20000)
-      len = get_data(p_out[(sent_version) ? p_out_s : ((p_out_s+1)%N_PACKETS)].data,
-        packet_len);
+    int i;
+    i = (sent_version) ? p_out_s : ((p_out_s+1)%N_PACKETS);
+    if (remote_term_version >= 20000) 
+      len = get_data(p_out[i].data,&type, packet_len);
     else
-      len = get_data(p_out[(sent_version) ? p_out_s : ((p_out_s+1)%N_PACKETS)].data,
+      len = get_data(p_out[i].data,&type,
         (packet_len <= out_mask) ? packet_len : out_mask);
     SANITY(packet_len <= out_mask+1);
   }else { 
     len = old_len;
+    type = old_type;
     old_len = 0;
   }
 
@@ -113,17 +116,20 @@ void do_link_out(void) {
   }
 
   if (! sent_version) { 
-    sprintf((char *)p_out[p_out_s].data, "VERSION %lu %lx %s",
-      (unsigned long)VERSION, term_localaddr, term_localhost);
+    sprintf((char *)p_out[p_out_s].data, "VERSION %lu %lx %s %u",
+      (unsigned long)VERSION, term_localaddr, term_localhost, 
+      (unsigned int)geteuid());
     old_len = len;
+    old_type = type;
     len = strlen((char *)p_out[p_out_s].data) + 1;
+    type = 0;
   }
 
   p_out[p_out_s].timeout = 0;	/* Transmit right away */
   p_out[p_out_s].queue = &curr_client->queue;
   p_out[p_out_s].trans = 0;
   p_out[p_out_s].len = (len > 0) ? len : -len;
-  p_out[p_out_s].type = (sent_version) ? ((len > 0) ? (seven_bit_out ? 2 : 0) : 1) : 0;
+  p_out[p_out_s].type = type;
   DEBUG_LINK(stderr, "%s:Added pack %d to out Q %d\n", term_server, p_out_s,
     curr_client->number);
   p_out_s = (p_out_s + 1) % N_PACKETS;
@@ -218,8 +224,8 @@ void do_link_in(void) {
     } else if (p_in[p_in_e].type == 0) {	/* uncompressed 8 bit data */
       if (! remote_term_version) {
         memset(term_remotehost,0,sizeof(term_remotehost));
-        if(sscanf((char *)p_in[p_in_e].data, "VERSION %lu %lx %s",
-            &remote_term_version, &term_remoteaddr, term_remotehost) < 1) {
+        if(sscanf((char *)p_in[p_in_e].data, "VERSION %lu %lx %s %u",
+            &remote_term_version, &term_remoteaddr, term_remotehost, &term_remoteuser) < 1) {
           remote_term_version = 10800;
           put_data(p_in[p_in_e].data, p_in[p_in_e].len);
         }
@@ -231,6 +237,10 @@ void do_link_in(void) {
           strcpy(term_remotehost,"remotehost");
       } else    
         put_data(p_in[p_in_e].data, p_in[p_in_e].len);
+    } else if (p_in[p_in_e].type == 5) {        /* new seven bit data */
+      l = convert_to_8(tok_byte_width_in,p_in[p_in_e].len, p_in[p_in_e].data,
+        uncomp_buff);
+      put_data(uncomp_buff, l);
     } else {	/* seven bit data */
       l = s_2_e_buff(p_in[p_in_e].data, uncomp_buff,
 		     p_in[p_in_e].len);
@@ -248,7 +258,7 @@ void do_link_in(void) {
 /* This is where compression will eventually get done */
 /* For now , we just get some bytes */
 /* we are compressing now. */
-int get_data(un_char *b, int len) {
+int get_data(un_char *b, int *type, int len) {
   extern int tok_byte_width_out;
   int i, j, k;
   /* this is a horrible kludge, but without major reorganization, 
@@ -261,6 +271,7 @@ int get_data(un_char *b, int len) {
    * croutons.
    */
 				/* Get a byte, so we we have a client. */
+  *type = -1;
   k = get_client_byte();
   if ( k < 0 )			/* no data */
   	return 0;
@@ -288,6 +299,7 @@ int get_data(un_char *b, int len) {
             curr_client->compress -= curr_client->compress / 2;
           j = ( j + tok_byte_width_out - 1) / tok_byte_width_out;
         }
+        *type = 1;
         return -j;
       }
       break;
@@ -302,18 +314,33 @@ int get_data(un_char *b, int len) {
     DEBUG_SER(stderr,"%s:sevenbit packet\n",term_server);
     len = (len * 7) / 8;
     b[0] = 0;
-    j = e_2_s_put(b, (unsigned) k, 0);
-    while ((j>>3) < len) {
-      if ((i = get_client_byte()) < 0) 
-	break;
-      j = e_2_s_put(b, (unsigned) i, j);
+    if (remote_term_version < 20251) {
+      j = e_2_s_put(b, (unsigned) k, 0);
+      while ((j>>3) < len) {
+        if ((i = get_client_byte()) < 0) break;
+        j = e_2_s_put(b, (unsigned) i, j);
+      }
+      *type = 2;
+      return (j>>3) + 1;
+    }else {
+      for(j=0, i=k;i >= 0;i=get_client_byte()) {
+        b[j++]=i;
+        if (j >= len) break;
+      }
+      if ((j=convert_from_8(tok_byte_width_out,j,b,b)) < 0) {
+        *type = 0;
+        return -j;
+      }else {
+        *type = 5;
+        return j;
+      }
     }
-    return (j>>3) + 1;
   } else {			/* Else, just dump the data, we can */
 				/* handle it. */
     b[0] = k; /* put the first byte in the array */
     j = 1;
-  
+ 
+    *type = 0; 
     while (j < len) {
       i = get_client_byte();
       if (i < 0) break;
@@ -404,7 +431,7 @@ void ret_fail(struct Client *cl, int which, int fatal, char *p) {
     add_ret_buff_str(cl, which, p);
     add_ret_buff_str(cl, which, ": ");
   }
-  add_ret_buff_str(cl, which, strerror(olderrno));
+  add_ret_buff_str(cl, which, x__strerror(olderrno));
   add_ret_buff(cl, which, 0);
   if(!fatal){
     errno = olderrno;
@@ -499,7 +526,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
       }
       if (chmod(++a,k) < 0) {
         DEBUG_FP(stderr, "%s: chmod %s failed: %s\n",term_server, a,
-          strerror(errno));
+          x__strerror(errno));
         break;
       }
 
@@ -512,7 +539,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
         utb.modtime = (time_t) (mtime + now);
         if (utime(a,&utb) < 0) { 
           DEBUG_FP(stderr, "%s: utimes failed: %s\n",term_server,
-            strerror(errno));
+            x__strerror(errno));
           break;
         }
       }
@@ -548,7 +575,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
         }
       }else a = (char *)c;
 
-      if (cl->fd >=0) close(cl->fd);
+      if (cl->fd >=0) x__close(cl->fd);
 
       if (c[0] == C_OPEN) {
         chmod(++a,0600);
@@ -564,14 +591,15 @@ void do_control(int local , struct Client *cl, un_char *c) {
 
       if (cl->fd < 0) {
         ret_fail(cl,local,FATAL, "open() failed");
-        DEBUG_FP(stderr, "%s: open failed: %s\n",term_server,strerror(errno));
+        DEBUG_FP(stderr, "%s: open failed: %s\n",term_server,x__strerror(errno));
         break;
       }
+      set_nonblock(cl->fd);
       if (l != 0 && lseek(cl->fd, (off_t)l, 0) == -1) {
         ret_fail(cl,local, FATAL, "lseek failed");
         DEBUG_FP(stderr, "%s: lseek failed: %s\n",term_server,
-	  strerror(errno));
-        close(cl->fd);
+	  x__strerror(errno));
+        x__close(cl->fd);
         cl->fd = -1;
         break;
       }
@@ -608,7 +636,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
 	ret_fail(cl, local, ! FATAL, "stat() failed");
 	DEBUG_FP(stderr, "%s: stat() failed\n", term_server);
         if(c[0] == C_DOWNLOAD) {
-          close(cl->fd);
+          x__close(cl->fd);
           cl->fd = -1;
         }
 	break;
@@ -651,14 +679,14 @@ void do_control(int local , struct Client *cl, un_char *c) {
     if (unlink(c+1)) {
       ret_fail(cl, local, ! FATAL, "unlink failed");
       DEBUG_FP(stderr, "%s: unlink failed: %s\n", term_server,
-	strerror(errno));
+	x__strerror(errno));
     } else
       ret_ok(cl, local);
     break;
 
   case C_PTYEXEC:
   case C_EXEC:
-    if (cl->fd>=0) close(cl->fd);
+    if (cl->fd>=0) x__close(cl->fd);
     DEBUG_FP(stderr, "%s: %s on client %d (%s) \n", term_server,
 	     c[0]==C_PTYEXEC?"C_PTYEXEC":"C_EXEC", cl->number, c+1); 
     if (rshtype > 0) cl->fd = -5;
@@ -687,6 +715,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
       ret_fail(cl, local, FATAL, p);
       break;
     }
+    set_nonblock(cl->fd);
     DEBUG_FP(stderr, "%s: opened client\n", term_server);
     cl->type = T_WRFILE | T_RDFILE;
     cl->cl_type = CL_CHILD;
@@ -706,27 +735,26 @@ void do_control(int local , struct Client *cl, un_char *c) {
       DEBUG_FP(stderr, "%s: c_bind %u %d\n", term_server, port, backlog);
 
       if ((s = bind_tcp_listen(port,backlog)) < 0) {
-	errno = 0;
 	ret_fail(cl, local , FATAL, "bind_tcp_listen() failed");
-	DEBUG_FP(stderr, "%s:bind_tcp_listen failed (%d): %s\n", term_server, port,
-	  strerror(errno));
+	DEBUG_FP(stderr, "%s:bind_tcp_listen failed (%d):%s\n", term_server,
+           port,x__strerror(errno));
       }
       if (! port) {
         struct sockaddr_in addr_in;
         int k=sizeof(addr_in);
 
-        if (getsockname(s, (struct sockaddr *)&addr_in, &k) < 0) {
+        if (x__getsockname(s, (struct sockaddr *)&addr_in, &k) < 0) {
           DEBUG_FP(stderr, "%s:getsockname failed: %s\n", term_server,
-	    strerror(errno));
+	    x__strerror(errno));
         }else {
   	  DEBUG_FP(stderr, "%s:sockname returned %s\n", term_server,
             sockaddr_to_str((struct sockaddr *)&addr_in, 0));
           port = ntohs(addr_in.sin_port);
         }
       }
-      set_nonblock(s);
-      if (cl->fd>=0) close(cl->fd);
+      if (cl->fd>=0) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_BOUND;
       cl->type = T_RDFILE | T_WRFILE;
       cl->state = 1;
@@ -765,16 +793,15 @@ void do_control(int local , struct Client *cl, un_char *c) {
         clients[s].type |= T_RDFILE | T_WRFILE;  
 
 				/* try the actual accept(). */
-      s = accept(clients[s].fd , (struct sockaddr *) &addr_in, &ain);
+      s = x__accept(clients[s].fd , (struct sockaddr *) &addr_in, &ain);
       if (s < 0) {
 	ret_fail(cl, local, FATAL, "Accept failed");
-        DEBUG_FP(stderr, "%s: accept failed: %s\n", term_server, strerror(errno));
+        DEBUG_FP(stderr, "%s: accept failed: %s\n", term_server, x__strerror(errno));
 	break;
       }
 
       memcpy(&cl->peername,&addr_in,sizeof(cl->peername));
 
-      set_nonblock(s);
       add_ret_buff(cl, local,SWITCH);
       add_ret_buff(cl, local,SWITCH-3);
       add_ret_buff(cl, local,I_OK);
@@ -782,102 +809,129 @@ void do_control(int local , struct Client *cl, un_char *c) {
       add_ret_buff(cl, local,0);
 
       DEBUG_FP(stderr,"%s:got c_accept\n", term_server);
-      if (cl->fd>=0) close(cl->fd);
+      if (cl->fd>=0) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_SOCKET;
       DEBUG_FP(stderr, "%s:name is %s\n", term_server, (char *)(c+1));
       cl->type = T_RDFILE | T_WRFILE;
       cl->state = 1;
     }
     break;
-#ifndef X_STREAMS_PIPE
+#if !defined(X_STREAMS_PIPE) && !defined(SVR3)
   case C_X_SERVER:
-#ifdef SVR3
-    {
-      int s;
-      struct hostent *hostaddr;
-      char *display;
-      char host[80];
-      int  screen = 0;
-      struct sockaddr_in sock_in;
-      int i;
-
-      /*
-       * This code will use the environment variable DISPLAY that was in
-       * effect at the time "term" was started to direct all X connections
-       * to.   It only works for INET domain sockets
-       */
-      display = getenv("DISPLAY");
-      if (display == NULL)
-	display = (char *)(c+1);
-      for (i = 0; display[i] && display[i] != ':'; i++) {
-	host[i] = display[i];
-      }
-      host[i] = '\000';
-      if (host[i] == ':')
-	screen = atoi(&host[i+1]);
-      
-      hostaddr = host_lookup(host,0,AF_INET,0,NULL);
-      
-      if (hostaddr == NULL) {
-	ret_fail(cl, local, FATAL, "Failed to find DISPLAY host");
-	break;
-      }
-
-      if ((s = socket(AF_INET, SOCK_STREAM, PF_INET)) < 0) {
-	perror("Failed to allocate socket for X display");
-	ret_fail(cl, local, FATAL, "socket allocation failed");
-	break;
-      }
-      
-      sock_in.sin_family = AF_INET;
-      bcopy((char *) hostaddr->h_addr_list[0], (char *) &sock_in.sin_addr,
-	    hostaddr->h_length);
-      sock_in.sin_port = htons(6000+screen);
-
-      fprintf(stderr, "Attempting connection to host %d.%d.%d.%d\n",
-	      sock_in.sin_addr.s_addr & 0xff,
-	      (sock_in.sin_addr.s_addr >> 8) & 0xff,
-	      (sock_in.sin_addr.s_addr >> 16) & 0xff,
-	      (sock_in.sin_addr.s_addr >> 24) & 0xff);
-      if (connect(s, &sock_in, sizeof(sock_in))) {
-	perror("Connection attempt to X server failed");
-	ret_fail(cl, local, FATAL, "Failed to connect to display");
-	break;
-      }
-      fprintf(stderr, "Connection was successful\n");
-
-      set_nonblock(s);
-      DEBUG_FP(stderr, "%s:got c_x_socket\n", term_server);
-      if (cl->fd>=0) close(cl->fd);
-      cl->fd = s;
-      cl->cl_type = CL_SOCKET;
-      DEBUG_FP(stderr, "%s:name is %s\n", term_server, (char *)(c+1));
-      cl->type = T_RDFILE | T_WRFILE;
-      cl->state = 1;
-      ret_ok(cl, local);
-    }
-    break;
-#endif
 #endif
   case C_SOCKET:
     DEBUG_FP(stderr, "%s: c_socket %s\n", term_server, (char *)(c+1));
     {
 
       int s;
-      
-      s = open_unix((char *)(c+1));
-      if (s < 0) {
-	ret_fail(cl, local, FATAL, "open_unix() failed");
-	DEBUG_FP(stderr, "%s:open_unix failed: %s\n", term_server,
-	  strerror(errno));
+
+      if (! strcmp((char *)(c+1),REMOTE_X)) {
+	int screen=0;
+	char *display, path[270];
+
+/* This allows us to connect to the DISPLAY specified by DISPLAY */
+/* at the time the term connection was formed. */
+
+        if ((display = getenv("DISPLAY"))) {
+	  int i;
+          for (i = 0; display[i] && display[i] != ':'; i++);
+          if (display[i] == ':')
+ 	    screen = atoi(&display[i+1]);
+        }
+	sprintf(path,"%s/%s%d",LOCAL_X_DIR,LOCAL_X_NAME,screen);
+	s = open_unix(path);
+      }else {
+	s = open_unix((char *)(c+1));
+      }
+      if (s >= 0) {
+        DEBUG_FP(stderr,"%s:got c_socket\n", term_server);
+        if (cl->fd>=0) x__close(cl->fd);
+        cl->fd = s;
+        set_nonblock(cl->fd);
+        cl->cl_type = CL_SOCKET;
+        DEBUG_FP(stderr, "%s:name is %s\n", term_server, (char *)(c+1));
+        cl->type = T_RDFILE | T_WRFILE;
+        cl->state = 1;
+        ret_ok(cl, local);
 	break;
       }
+#if defined(X_STREAMS_PIPE) || !defined(X_TCP_SOCKET)
+      else{
+	ret_fail(cl, local, FATAL, "open_unix() failed");
+	DEBUG_FP(stderr, "%s:open_unix failed: %s\n", term_server,
+	  x__strerror(errno));
+        break;
+      }
+#endif
+    }
+#ifndef X_STREAMS_PIPE
+#ifdef SVR3
+  case C_X_SERVER:
+#endif
+    {
+      int s;
+      struct hostent *hostaddr = NULL;
+      char *display;
+      char host[259];
+      int  screen = 0;
+      struct sockaddr_in sock_in;
+      int i = 0;
 
-      set_nonblock(s);
-      DEBUG_FP(stderr,"%s:got c_socket\n", term_server);
-      if (cl->fd>=0) close(cl->fd);
+      /*
+       * This code will use the environment variable DISPLAY that was in
+       * effect at the time "term" was started to direct all X connections
+       * to.   It only works for INET domain sockets
+       */
+      if ( (display = getenv("DISPLAY"))) {
+        for (i = 0; display[i] && display[i] != ':'; i++) 
+ 	  host[i] = display[i];
+        if (display[i] == ':')
+          screen = atoi(&display[i+1]);
+      }
+      host[i] = 0;
+
+      if (*host) {
+        struct in_addr *addr_in;
+
+        hostaddr = host_lookup(host,0,AF_INET,0,NULL);
+        if (! hostaddr) {
+  	  ret_fail(cl, local, FATAL, "Failed to find DISPLAY host");
+  	  DEBUG_FP(stderr, "Failed to find DISPLAY host %s", host);
+	  break;
+        }
+	addr_in = (struct in_addr *) hostaddr->h_addr;
+  	sock_in.sin_addr.s_addr = addr_in->s_addr;
+      }else {
+	sock_in.sin_addr.s_addr = htonl(term_localaddr);
+      }
+
+      if ((s = x__socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	ret_fail(cl, local, FATAL, "socket allocation failed");
+	x__perror("Failed to allocate socket for X display");
+	break;
+      }
+      
+      sock_in.sin_family = AF_INET;
+      sock_in.sin_port = htons(6000+screen);
+
+      DEBUG_FP(stderr, "Attempting connection to host %lu.%lu.%lu.%lu\n",
+	      sock_in.sin_addr.s_addr & 0xff,
+	      (sock_in.sin_addr.s_addr >> 8) & 0xff,
+	      (sock_in.sin_addr.s_addr >> 16) & 0xff,
+	      (sock_in.sin_addr.s_addr >> 24) & 0xff);
+      if (x__connect(s, (struct sockaddr *)&sock_in, sizeof(sock_in))) {
+	ret_fail(cl, local, FATAL, "Failed to connect to display");
+	DEBUG_FP(stderr,"Connection attempt to X server failed");
+	break;
+      }
+      DEBUG_FP(stderr, "Connection was successful\n");
+
+      DEBUG_FP(stderr, "%s:got c_x_socket\n", term_server);
+      if (cl->fd>=0) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_SOCKET;
       DEBUG_FP(stderr, "%s:name is %s\n", term_server, (char *)(c+1));
       cl->type = T_RDFILE | T_WRFILE;
@@ -885,7 +939,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
       ret_ok(cl, local);
     }
     break;
-#ifdef X_STREAMS_PIPE
+#else /* X_STREAMS_PIPE */
   case C_X_SERVER:
     DEBUG_FP(stderr, "%s: c_x_server %s\n", term_server, (char *)(c+1));
     {
@@ -910,14 +964,14 @@ void do_control(int local , struct Client *cl, un_char *c) {
       if (s < 0) {
 	ret_fail(cl, local, FATAL, "X connection failed");
 	DEBUG_FP(stderr, "%s: x connection failed: %s\n", term_server,
-	  strerror(errno));
+	  x__strerror(errno));
 	break;
       }
 
-      set_nonblock(s);
       DEBUG_FP(stderr,"%s:got c_x_server\n", term_server);
-      if (cl->fd>=0) close(cl->fd);
+      if (cl->fd>=0) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_SPIPE;
       DEBUG_FP(stderr, "%s:name is %s\n", term_server, (char *)(c+1));
       cl->type = T_RDFILE | T_WRFILE;
@@ -941,10 +995,10 @@ void do_control(int local , struct Client *cl, un_char *c) {
 
       if (old && cl->fd >= 0) {
         s = cl->fd;
-      }else if ((s = socket(AF_INET, SOCK_STREAM, 0 )) < 0) {
+      }else if ((s = x__socket(AF_INET, SOCK_STREAM, 0 )) < 0) {
         ret_fail(cl, local, FATAL, "Socket() failed");
         DEBUG_FP(stderr, "%s: Socket() failed: %s\n", term_server, 
-          strerror(errno));
+          x__strerror(errno));
         break;
       }
 
@@ -952,7 +1006,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
         errno = 0;
         ret_fail(cl, local, FATAL, "Can't get local address");
         DEBUG_FP(stderr,"%s: Can't get local address\n", term_server);
-        close(s);
+        x__close(s);
         break;
       }
  
@@ -963,27 +1017,24 @@ void do_control(int local , struct Client *cl, un_char *c) {
 
       DEBUG_FP(stderr, "%s: connecting to to %s\n", term_server,
 	  sockaddr_to_str(addr,INADDR_ANY));
-      if (connect(s,addr,sizeof(struct sockaddr))<0) {
+      if (x__connect(s,addr,sizeof(struct sockaddr))<0) {
 #ifndef USE_CONNBLOCK
         if (errno != EINPROGRESS && errno != ERR_BLOCK) {
 #endif
           ret_fail(cl,local, FATAL, "connect() failed");
           DEBUG_FP(stderr, "%s: connect() failed: %s\n", term_server,
-            strerror(errno));
-          close(s);
+            x__strerror(errno));
+          x__close(s);
           break;
 #ifndef USE_CONNBLOCK
         }
 #endif
       }
       
-#ifdef USE_CONNBLOCK
-      set_nonblock(s);
-#endif
-
       DEBUG_FP(stderr,"%s:got c_port\n", term_server);
-      if (cl->fd>=0 && ! old) close(cl->fd);
+      if (cl->fd>=0 && ! old) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_SOCKET;
       cl->type = T_RDFILE | T_WRFILE;
 #ifndef USE_CONNBLOCK
@@ -1018,13 +1069,13 @@ void do_control(int local , struct Client *cl, un_char *c) {
         DEBUG_FP(stderr,"%s: Client out of range\n",term_server);
 	break;
       }
-      if (listen(clients[k].fd, queue) < 0) { /* If we can't listen... */
+      if (x__listen(clients[k].fd, queue) < 0) { /* If we can't listen... */
 #if 0
         ret_fail(cl, local, ! FATAL, "listen() failed");
 #endif
         DEBUG_FP(stderr,"%s: listen() failed: %s\n", term_server, 
-          strerror(errno));
-        close(clients[k].fd);
+          x__strerror(errno));
+        x__close(clients[k].fd);
         clients[k].fd = -1;
         break;
       }
@@ -1133,10 +1184,10 @@ void do_control(int local , struct Client *cl, un_char *c) {
         break;
       }
       k=sizeof(addr_in);
-      if (getsockname(s, (struct sockaddr *)&addr_in, &k) < 0) {
+      if (x__getsockname(s, (struct sockaddr *)&addr_in, &k) < 0) {
         ret_fail(cl, local, FATAL, "getsockname() failed");
         DEBUG_FP(stderr, "%s:getsockname failed: %s\n", term_server,
-	  strerror(errno));
+	  x__strerror(errno));
         break;
       }
       
@@ -1147,9 +1198,9 @@ void do_control(int local , struct Client *cl, un_char *c) {
       add_ret_buff_str(cl,local,sockaddr_to_str((struct sockaddr *)&addr_in,1));
       add_ret_buff(cl, local,0);
 
-      set_nonblock(s);
-      if (cl->fd>=0) close(cl->fd);
+      if (cl->fd>=0) x__close(cl->fd);
       cl->fd = s;
+      set_nonblock(cl->fd);
       cl->cl_type = CL_BOUND;
       cl->type = T_RDFILE | T_WRFILE;
       cl->state = 1;
@@ -1192,10 +1243,10 @@ void do_control(int local , struct Client *cl, un_char *c) {
       }
       
       k=sizeof(addr);
-      if (getsockname(s, &addr, &k) < 0) {
+      if (x__getsockname(s, &addr, &k) < 0) {
         ret_fail(cl, local, ! FATAL, "getsockname() failed");
         DEBUG_FP(stderr, "%s:getsockname failed: %s\n", term_server,
-	  strerror(errno));
+	  x__strerror(errno));
         break;
       }
       
@@ -1251,13 +1302,13 @@ void do_control(int local , struct Client *cl, un_char *c) {
       }
       
       k=sizeof(addr);
-      if (addr_in->sin_family != ~0) {
+      if (~addr_in->sin_family) {
         if (k > sizeof(struct sockaddr_in)) k = sizeof(struct sockaddr_in);
         memcpy((struct sockaddr_in *)&addr, addr_in, k);
-      }else if (getpeername(s, &addr, &k) < 0) {
+      }else if (x__getpeername(s, &addr, &k) < 0) {
         ret_fail(cl, local, ! FATAL, "getpeername() failed");
         DEBUG_FP(stderr, "%s: getpeername failed: %s\n", term_server,
-	  strerror(errno));
+	  x__strerror(errno));
         break;
       }
       
@@ -1286,7 +1337,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
         uname(&unam);
         strcpy(hostname, unam.nodename);
 #else
-        gethostname(hostname, sizeof(hostname));
+        x__gethostname(hostname, sizeof(hostname));
 #endif /* SYSV */
       }
      
@@ -1332,6 +1383,7 @@ void do_control(int local , struct Client *cl, un_char *c) {
       } 
  
       onetime_term[j].socket = bind_tcp_listen(0,5);
+      onetime_term[j].errors = 0;
       onetime_term[j].timeout = current_time + term_inc * 30;
 	 /* allow 30 seconds for connection */
 
@@ -1343,11 +1395,11 @@ void do_control(int local , struct Client *cl, un_char *c) {
       }
 
       k=sizeof(addr_in);
-      if (getsockname(onetime_term[j].socket, (struct sockaddr *)&addr_in, &k) < 0) {
+      if (x__getsockname(onetime_term[j].socket, (struct sockaddr *)&addr_in, &k) < 0) {
         ret_fail(cl, local, FATAL, "getsockname() failed");
         DEBUG_FP(stderr, "%s: getsockname failed: %s\n", term_server,
-	  strerror(errno));
-        close(onetime_term[j].socket);
+	  x__strerror(errno));
+        x__close(onetime_term[j].socket);
         onetime_term[j].socket = -1;
         break;
       }
@@ -1382,14 +1434,14 @@ void do_control(int local , struct Client *cl, un_char *c) {
         ret_fail(cl, local, ! FATAL, "Client out of range");
         DEBUG_FP(stderr,"client out of range\n");
         break;
-      }else if( (s = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
+      }else if( (s = x__socket(AF_INET,SOCK_DGRAM,0)) < 0) {
         ret_fail(cl,local,! FATAL,"dgram socket() failed");      /* 0 ? is it fatal? */
         DEBUG_UDP(stderr, "%s: dgram socket failed: %s\n", term_server,
-	  strerror(errno));
+	  x__strerror(errno));
         break;
       }
 
-      if( clients[k].fd >= 0) close(clients[k].fd);
+      if( clients[k].fd >= 0) x__close(clients[k].fd);
 
       clients[k].fd = s;
       clients[k].type = T_WRFILE | T_RDFILE | T_UDP;
@@ -1435,10 +1487,10 @@ void do_control(int local , struct Client *cl, un_char *c) {
         break;
       }
 
-      if( bind(clients[k].fd,addr,sizeof(struct sockaddr)) < 0) {
+      if( x__bind(clients[k].fd,addr,sizeof(struct sockaddr)) < 0) {
         ret_fail(cl,local, ! FATAL,"bind() failed");
         DEBUG_FP(stderr,"term: bind() failed: %s: %s\n", (char *)c+1,
-          strerror(errno));
+          x__strerror(errno));
         break;
       }
       ret_ok(cl,local);
@@ -1518,10 +1570,11 @@ void init_client(struct Client *cl) {
   cl->name[0] = 0;
   cl->parent = -1;
   cl->peername.sin_port = ~0;
-  cl->peername.sin_addr.s_addr = term_localaddr;
+  cl->peername.sin_addr.s_addr = htonl(term_localaddr);
   cl->peername.sin_family = ~0;
+  cl->owner = -1;
   if (cl->fd > 0) {
-    close(cl->fd);
+    x__close(cl->fd);
     cl->fd = -1;
   } 
   clear_buffers(cl);
@@ -1775,7 +1828,7 @@ int get_client_byte() {
 	return -1;
       
       if (next > 1023) {
-        if (curr_client->fd >= 0) close(curr_client->fd);
+        if (curr_client->fd >= 0) x__close(curr_client->fd);
       }else {
         control[next++] = i;
       }
