@@ -35,6 +35,9 @@ static un_char rewind_buffer[264];
 void do_ack(unsigned int);
 void send_ack(unsigned int);
 int check_match(int, int);
+/* adjust_timeout changes the current timeout value based on Jacobson's */
+/* modified version of RFC 793 */
+void adjust_timeout(unsigned int rtt, int trans);
 
 /*----------------------------------------------------------------------*/
 /* Various ring buffers */
@@ -66,7 +69,7 @@ void serial_init() {
 		    
   p_in_e =
   p_in_num = 0;
-
+  
   for (i = 0; i < N_PACKETS;++i) {
     p_out[i].type = -1;
     p_in[i].type = -1;
@@ -200,8 +203,8 @@ void do_serial_out(int force) {
   unsigned int l;
 				/* port is ready for writing. see */
 				/* what's available. */ 
-  
-  if (!p_out_num || serial_out.size || inhabit_send)
+
+  if ( !p_out_num || serial_out.size || inhabit_send)
     return;			/* Hmm. How did this get called. */
   
 				/* Check for timeouts.. */
@@ -249,7 +252,9 @@ void do_serial_out(int force) {
     window_size = 1;  /* This allows for a complete recovery. */
   }
   p_out[l].timeout = current_time;
-  if (!p_out[l].trans++) 
+  if (auto_retrain) 
+    adjust_timeout(current_time-p_out[l].q_time, p_out[l].trans);
+  if (!p_out[l].trans++)
     *p_out[l].queue = (*p_out[l].queue > 0) ? (*p_out[l].queue - 1) : 0;
 }
 
@@ -264,7 +269,7 @@ void do_serial_in() {
     curr_p_index = 0, 
     curr_p_type = 0,
     curr_p_num = 0;
-  static un_char header[4];
+  static un_char header[7];
   
   if (p_in_num > N_PACKETS/2 || !serial_in.size)
     return; /* packet window is full */
@@ -329,6 +334,39 @@ void do_serial_in() {
 	break;			/* more to read yet */
 				/* Ok. We have the whole header. Check */
 				/* the checksum */ 
+      if (header[0] == 'E') {	/* Got a superack.  Need to do special error */
+  				/* checking for it.  Not a standard packet */
+        if (curr_p_index < 7)
+          break;
+        i = header[1] + header[2] + header[3] + header[4]; 
+        if (i != header[5]) {
+          DEBUG_CHECK(stderr, "Bad checksum on a superack packet\n");
+          curr_p_stage = 0;
+          break;
+        }
+        i = update_crc(update_crc(update_crc(update_crc(update_crc(update_crc(
+            0, (unsigned int) header[0] & (unsigned int) in_mask),
+               (unsigned int) header[1] & (unsigned int) in_mask),
+               (unsigned int) header[2] & (unsigned int) in_mask),
+               (unsigned int) header[3] & (unsigned int) in_mask),
+               (unsigned int) header[4] & (unsigned int) in_mask),
+               (unsigned int) header[5] & (unsigned int) in_mask);
+        if (!check_match( i & in_mask, (int) header[6])) {
+          DEBUG_CHECK(stderr, "%s:invalid check calc on super ack %x, read %x\n",
+                      term_server, i, header[6]);
+          curr_p_stage=0;
+          break;
+        }
+        DEBUG_SER(stderr, "%s: got super ack:", term_server);
+        for (i=0; i<N_PACKETS; i++) 
+          if (header[1+(i / 8)] & (1 << (i % 8))) {
+            DEBUG_SER(stderr, "%d ", i);
+            do_ack((unsigned int) i);
+          }
+        DEBUG_SER(stderr, "\n");
+        curr_p_stage=0;
+        break;
+      }
       i = update_crc(update_crc(update_crc(0,
 		(unsigned int) header[0]& (unsigned int) in_mask),
 		(unsigned int) header[1]& (unsigned int) in_mask),
@@ -376,10 +414,10 @@ void do_serial_in() {
 	do_ack(header[1]);
 	curr_p_stage = 0;
 	break;
-      }			
+      }
 				/* see if we want the packet */
       if (!header[1] && remote_term_version >= 20000)
-        curr_p_len = 258;
+        curr_p_len = 2 + (1 << tok_byte_width_in);
       else	
         curr_p_len = header[1] + 2; /* get checksum as well. */
 
@@ -389,7 +427,7 @@ void do_serial_in() {
 				/* an old packet. just lose it */
         WARNING(stderr,"old packet %d received.  try raising your %s timeout.\n",
           i, remote ? "local" : "remote");
-        curr_p_num = N_PACKETS;
+	curr_p_num = N_PACKETS;
       }else {
         curr_p_num = header[2] % N_PACKETS;
         if (p_in[curr_p_num].type >= 0) {
@@ -398,13 +436,14 @@ void do_serial_in() {
 				/* check it */ 
 	  if (header[1] != p_in[curr_p_num].len) {
 				/* We have a different length !! */
-	    alert("Duplicate packet received with a different lengths.\n");
+	    alert("Duplicate packet received with a different lengths\n");
 				/* just ignore it for now. Will HAVE */
 				/* to be fixed */ 
 	  }else {
             WARNING(stderr,"duplicate packet received\n");
+				/* discard following data and checksum */
           }
-          curr_p_num = N_PACKETS;
+	  curr_p_num = N_PACKETS;
         }
       }
       curr_p_stage = 2;
@@ -474,7 +513,8 @@ void do_serial_in() {
 				/* meta-state used by term to check */
 				/* for characters that are getting */
 				/* eaten, or to do other things that */
-				/* aren't based on packets. */
+				/* aren't based on 
+				packets. */
       i = get_serial();
       if (i < 0) break;
 
@@ -509,14 +549,12 @@ void do_ack(unsigned int num) {
     --p_out_num;
   }
 
-	/* In case of timeouts, we need to dequeue all the packets befor */
+	/* In case of timeouts, we need to dequeue all the packets before */
 	/* we continue. */
   if(window_size == 1 && ((p_out_e + N_PACKETS) - p_out_num) % N_PACKETS <= 1 ) 
     window_size = window_size_max;
 }
 
-
-
 void send_ack(unsigned int num) {
 				/* yucky eh?? 4 bytes to transmit 1. */
 				/* Oh well. I fix it in some other version */ 
@@ -537,3 +575,20 @@ int check_match(int check, int calc) {
     (((check >> 8) & 127) == ((calc>>8) & 127));
 }
 
+void adjust_timeout(unsigned int rtt, int trans)
+{
+  static int A=0;
+  static int D=0;
+  int Err;
+
+  if (!A) 
+    A = packet_timeout/2;
+  if (!D)
+    D = packet_timeout/8;
+  Err = (int)rtt-A;
+  A += Err/8;
+  D += (abs(Err) - D)/4;
+  packet_timeout = A + (4 * D);
+  if (trans) 
+    packet_timeout+=packet_timeout/2;
+}
