@@ -28,6 +28,9 @@ static int local_options ( char opt, char *term_optarg )
 int main(int argc, char *argv[]) {
   int s;
 
+#ifdef SOCKS
+  SOCKSinit(argv[0]);
+#endif
   (void) client_options(argc, argv,"vhn",local_options);
 
   use_term_command(PRIVILEGED);
