@@ -5,6 +5,9 @@ int main(int argc, char *argv[]) {
   int first, s, new, port;
   char buff[10];
 
+#ifdef SOCKS
+  SOCKSinit(argv[0]);
+#endif
   first = client_options(argc, argv);
   printf("first is %d\n", first);
 
@@ -26,6 +29,6 @@ int main(int argc, char *argv[]) {
   }
   write(new, "Hello.\n", 8);
   sleep(2);
-  close(new);
-  close(s);
+  x__close(new);
+  x__close(s);
 }
