@@ -15,19 +15,21 @@
 #define N_PACKETS       32
 #define SWITCH		'@'
 
-#define MAX_TYPE	5 /* There are 5 different types of packets */
+#define MAX_TYPE	6 /* There are 6 different types of packets */
 /* these are: */
-/*	uncompress packet */
-/* 	compressed packet */
-/*      packed (sevenbit) packet */
-/*	ack packet */
-/*	super ack packet */
+/* 0	uncompress packet */
+/* 1	compressed packet */
+/* 2    packed (old style sevenbit) packet */
+/* 3	ack packet */
+/* 4	super ack packet */
+/* 5    packed (sevenbit) packet */
 
 /* main.c */
 extern unsigned long current_time, packet_timeout, remote_term_version,
   term_localaddr, term_remoteaddr;
 
 extern char term_localhost[256], term_remotehost[256];
+extern unsigned int term_remoteuser;
 extern int
 	remote,
 	clients_waiting,
@@ -46,6 +48,7 @@ extern int
 	block_size,
         max_cps,
         auto_retrain,
+        use_term_socket,
         stat_comp_in,
         stat_comp_out,
         stat_modem_in,
@@ -101,6 +104,7 @@ extern struct Client {
     int fd;			/* socket file descriptor. */
     int priority;		/* How many packets may be queued */
     int queue;
+    int old_timeout;		/* The timeout before this one (Debugging) */
     int cl_type;		/* What type of fd are we watching. */
     int type;			/* dumb or smart */
 				/* This basically gets down to being */
@@ -128,6 +132,7 @@ extern struct Client {
     int udp_size; 		/* size of outgoing udp packets */
     int parent;			/* This is needed for udp sockets, so they  */
 				/* have some way of knowing when to close. */
+    int owner;
 
     struct sockaddr_in peername; /* This is where the socket is connected from */
 
@@ -137,8 +142,10 @@ extern int num_clients;
 extern int do_shutdown;
 extern struct Onetime {
     int socket;
+    int owner;
+    int errors;
     unsigned long timeout;
-    } onetime_term[MAX_CLIENTS];
+    } onetime_term[MAX_CLIENTS+1];
 
 /* serial.c */
 struct Packet_out {
@@ -146,6 +153,7 @@ struct Packet_out {
     int type;		/* flag for compressed or not */
     int len;
     unsigned long timeout;	/* What time it will timeout */
+    unsigned long q_time;	/* What time the packet was queued */
     int trans;
     int *queue;
     };
@@ -173,6 +181,9 @@ extern void serial_init(void);
 extern int get_client_byte(void);
 extern char *term_server;
 
+extern int tok_byte_mask_in, tok_byte_width_in;
+extern int tok_byte_mask_out, tok_byte_width_out;
+
 #if 1
 #define noise(a) do_noise(a)
 #define alert(b) do_alert(b)
@@ -227,7 +238,7 @@ int e_2_s_put(un_char *out, un_char data, int key);
 int s_2_e_buff(un_char *data, un_char *out, int len);
 
 /* utils.c */
-char *strerror(int errno);
+char *x__strerror(int errno);
 char *str_version(unsigned long int version);
 unsigned short file_crc(char *fname, long nbytes);
 
