@@ -16,7 +16,7 @@ void lose_ctty(void) {
     int slavefd;
     if ((slavefd = open ("/dev/tty", O_RDWR, 0)) >= 0) {
       ioctl (slavefd, TIOCNOTTY, (char *) 0);
-      close (slavefd);
+      x__close (slavefd);
     }
   }
 #endif
@@ -32,12 +32,6 @@ void lose_ctty(void) {
   setpgrp(mypid, 0); }
 #endif
 #endif
-    
-#ifdef USE_BSDJOBS
-  { int mypid;
-  mypid = getpid();
-  tcsetpgrp(0, mypid); }
-#endif
 }
 
 #ifdef USE_TERMIOS
@@ -179,7 +173,8 @@ void terminal_restore(int fd,int hangup)
   ioctl(fd, TIOCLSET, &o_lmode);
   ioctl(fd, TIOCSLTC, &o_ltchars);
   if (hangup) {
-    struct sgttyb disconnect = o_ttyb;
+    struct sgttyb disconnect;
+    disconnect = o_ttyb;
     disconnect.sg_ispeed = B0;
     disconnect.sg_ospeed = B0;
     ioctl(fd, TIOCSETP, &disconnect);
