@@ -11,6 +11,7 @@
  * figured out how to make term_bind network invisible yet.
  */
 
+#define I_ARGS
 #define I_IOCTL
 #define I_SYS
 #define I_GETOPT
@@ -177,6 +178,10 @@ static void close_fd(int fd,int how){
               sprintf(term_error, "Term: C_CLOSE 1 %d failed: %s",sockinfo[i].client,
                 command_result);
               print_err(1);
+              if (j < -1) {
+                x__close(s);
+                s = -1;
+              }
             }
           }
         }
@@ -352,7 +357,11 @@ static int term_udpsocket(int S) {
     tmp_errno = ~0;
     sprintf(term_error,"Term: C_STATS -6 failed: %s",command_result);
     print_err(1);
-    close(s);
+    x__close(s);
+    if (i < -1) {
+      x__close(s);
+      s = -1;
+    }
     return MAX_CLIENTS;
   }
   client = atoi(command_result);
@@ -371,7 +380,8 @@ static int term_udpsocket(int S) {
     print_err(1);
   }
 
-  close(s);
+  if (i < -1) term_do_exit();
+  x__close(s);
 
   if (i < 0) return MAX_CLIENTS;
 
@@ -393,6 +403,7 @@ static int term_udpsocket(int S) {
   }
 
   if (i < 0) {
+    if (i < -1) term_do_exit();
     if (S >= 0) 
       term_close(S);
     return MAX_CLIENTS;
@@ -407,7 +418,7 @@ void term_do_exit(void) {
   int k;
   for(k=0;k<MAX_CLIENTS;k++)
     if (sockinfo[k].sock > 0) term_close(k);
-  if (Sock >= 0) close(Sock);
+  if (Sock >= 0) x__close(Sock);
   Sock = -1;
   return;
 }
@@ -416,7 +427,7 @@ void term_do_exit(void) {
 
 char *term_strerror(int errnum) {
   char *s;
-  s=strerror(errnum);
+  s=x__strerror(errnum);
   if (! use_term || errnum != ~0) return s;
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -430,7 +441,7 @@ char *term_strerror(int errnum) {
 
 void term_perror(char *message) {
   if (! use_term || errno != ~0) {
-    perror(message);
+    x__perror(message);
     return;
   }
   if (remote_connect < 0) {
@@ -439,7 +450,7 @@ void term_perror(char *message) {
   }
   errno = ~0;
   if (! term_error[0]) {
-    perror(message);
+    x__perror(message);
     return;
   }
   fprintf(stderr,"%s: %s\n",message,term_error);
@@ -450,9 +461,9 @@ void term_perror(char *message) {
  */
 
 int term_getpeername(int S, struct sockaddr *name, int *namelen){
-  int i;
+  int i,j;
 
-  if(! use_term) return getpeername(S,name,namelen);
+  if(! use_term) return x__getpeername(S,name,namelen);
 
   if(S<0) return -1;
 
@@ -479,13 +490,14 @@ int term_getpeername(int S, struct sockaddr *name, int *namelen){
       tmp_errno = ~0;
       sprintf(term_error, "Term 4: %s", command_result);
       print_err(1);
-      return getpeername(S,name,namelen);
+      return x__getpeername(S,name,namelen);
     }
-    if(send_command(Sock, C_GETPEERNAME, 0, "%d", sockinfo[i].client) < 0) {
+    if((j=send_command(Sock, C_GETPEERNAME, 0, "%d", sockinfo[i].client)) < 0) {
       tmp_errno = ~0;
       sprintf(term_error, "C_GETPEERNAME: %s", command_result);
       print_err(1);
-      return getpeername(S,name,namelen);
+      if (j < -1) term_do_exit();
+      return x__getpeername(S,name,namelen);
     }
 
     if(name != NULL && namelen != NULL){
@@ -497,7 +509,7 @@ int term_getpeername(int S, struct sockaddr *name, int *namelen){
     }
   }
   if (use_term < 1)
-    return getpeername(S,name,namelen);
+    return x__getpeername(S,name,namelen);
   errno = ENOTCONN;
   return -1;
 }    
@@ -508,11 +520,11 @@ int term_getpeername(int S, struct sockaddr *name, int *namelen){
  */
 
 int term_getsockname(int S, struct sockaddr *name, int *namelen){
-  int i,j = -1;
+  int i,j = -1, k;
   struct hostent *hp;
   struct sockaddr_in *name_in;
 
-  if(! use_term) return getsockname(S,name,namelen);
+  if(! use_term) return x__getsockname(S,name,namelen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -526,7 +538,7 @@ int term_getsockname(int S, struct sockaddr *name, int *namelen){
   }
 
   if (use_term < 1) {
-    if ((j=getsockname(S,name,namelen)) < 0) return j;
+    if ((j=x__getsockname(S,name,namelen)) < 0) return j;
   }
   if (S < 0) {
     errno = EBADF;
@@ -547,7 +559,7 @@ int term_getsockname(int S, struct sockaddr *name, int *namelen){
         uname(&unam);
         strcpy(hostname, unam.nodename); }
 #else
-      gethostname (hostname, sizeof(hostname));
+      x__gethostname (hostname, sizeof(hostname));
 #endif
       if (isdigit(hostname[0])) {
         name_in->sin_addr.s_addr = inet_addr(hostname);
@@ -576,10 +588,11 @@ int term_getsockname(int S, struct sockaddr *name, int *namelen){
     print_err(1);
     return j;
   }
-  if (send_command(Sock, C_GETSOCKNAME, 0, "%d", sockinfo[i].client) < 0) {
+  if ((k=send_command(Sock, C_GETSOCKNAME, 0, "%d", sockinfo[i].client)) < 0) {
     tmp_errno = ~0;
     sprintf(term_error, "C_GETSOCKNAME: %d %d : %s", S, sockinfo[i].client, command_result);
     print_err(1);
+    if (k < -1) term_do_exit();
     return j;
   }
 
@@ -606,11 +619,16 @@ int term_getsockname(int S, struct sockaddr *name, int *namelen){
  * than that, the original hostnames will be replaced.)
  */
 
+#ifndef linux
 struct hostent *term_gethostbyname(char *host){
+#else
+struct hostent *term_gethostbyname(const char *host){
+#endif
+  int i;
   static char hostname[259];
   static struct hostent *hp;
 
-  if(! use_term) return host_lookup(host,0,AF_INET,0,NULL);
+  if(! use_term) return host_lookup((char *)host,0,AF_INET,0,NULL);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -624,13 +642,13 @@ struct hostent *term_gethostbyname(char *host){
   }
 
   if (host != NULL)
-    if (use_term < 1 && (hp=host_lookup(host,0,AF_INET,0,NULL)))
+    if (use_term < 1 && (hp=host_lookup((char *)host,0,AF_INET,0,NULL)))
       return hp;
 
 
   /* Copy the passed-in name, to make sure it doesn't get munged */
   if (host != NULL){
-    if ((hp=host_lookup(host,0,AF_INET,1,NULL))) return hp;
+    if ((hp=host_lookup((char *)host,0,AF_INET,1,NULL))) return hp;
     if(!strcmp(host,"127.0.0.254"))
       hostname[0] = '\0';
     else
@@ -642,26 +660,34 @@ struct hostent *term_gethostbyname(char *host){
   if (Sock < 0)
     if ((Sock = socket_connect_server(-1,term_server)) < 0) {
     tmp_errno = ~0;
-    sprintf(term_error, "Term: %s %s", host, command_result);
+    sprintf(term_error, "Term: %s %s", host ? host : "NULL", command_result);
     print_err(1);
     remote_connect = 0;
     return NULL;
   }
-  if (send_command(Sock, C_GETHOSTNAME, 0, "%s", hostname) < 0) {
+  if ((i=send_command(Sock, C_GETHOSTNAME, 0, "%s", hostname)) < 0) {
     tmp_errno = ~0;
     sprintf(term_error, "C_GETHOSTNAME: %s", command_result);
     print_err(1);
+    if (i < -1) term_do_exit();
     return NULL;
   }
-  return host_lookup(host,0,AF_INET,1,str_to_hostent(command_result));
+  if (host) 
+    return host_lookup((char *)host,0,AF_INET,1,str_to_hostent(command_result));
+  else
+    return str_to_hostent(command_result);
 }
 
+#ifndef linux
 struct hostent *term_gethostbyaddr(char *addr, int len, int type){
+#else
+struct hostent *term_gethostbyaddr(const char *addr, int len, int type){
+#endif
   struct hostent *hs;
   static char host[16];
 
   if(! use_term || type != AF_INET) {
-    return host_lookup(addr,len,type,0,NULL);
+    return host_lookup((char *)addr,len,type,0,NULL);
   }
 
   if (remote_connect < 0) {
@@ -675,7 +701,7 @@ struct hostent *term_gethostbyaddr(char *addr, int len, int type){
     print_err(32768);
   }
 
-  if(use_term < 1 && (hs=host_lookup(addr, len, AF_INET,0,NULL)))
+  if(use_term < 1 && (hs=host_lookup((char *)addr, len, AF_INET,0,NULL)))
     return hs;
 
   {
@@ -692,7 +718,7 @@ struct hostent *term_gethostbyaddr(char *addr, int len, int type){
 
 int term_shutdown(int fd,int how){
 
-  if(! use_term) return shutdown(fd,how);
+  if(! use_term) return x__shutdown(fd,how);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -706,7 +732,7 @@ int term_shutdown(int fd,int how){
   }
 
   close_fd(fd,how);
-  return shutdown(fd,how);
+  return x__shutdown(fd,how);
 }
 
 int term_close(int fd){
@@ -722,7 +748,7 @@ int term_close(int fd){
   }
 
   close_fd(fd,2);
-  return close(fd);
+  return x__close(fd);
 }   
 
 
@@ -731,9 +757,9 @@ int term_close(int fd){
  */
 
 int term_listen(int fd, int backlog){
-  int k;
+  int i,k;
 
-  if(! use_term) return listen(fd,backlog);
+  if(! use_term) return x__listen(fd,backlog);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -746,13 +772,13 @@ int term_listen(int fd, int backlog){
     print_err(32768);
   }
 
-  if(fd < 0) return listen(fd,backlog);
+  if(fd < 0) return x__listen(fd,backlog);
 
   for(k=0;k<MAX_CLIENTS;k++)
     if(fd==sockinfo[k].sock) break;
 
   if (k == MAX_CLIENTS) {
-    if (use_term < 1) return listen(fd,backlog);
+    if (use_term < 1) return x__listen(fd,backlog);
     errno = EOPNOTSUPP;
     return -1;
   }
@@ -764,11 +790,12 @@ int term_listen(int fd, int backlog){
     print_err(1);
     return -1;
   }
-  if (send_command(Sock,C_LISTEN,0,"%d %d",sockinfo[k].client,backlog)< 0) {
+  if ((i=send_command(Sock,C_LISTEN,0,"%d %d",sockinfo[k].client,backlog))< 0) {
     tmp_errno = ~0;
     sprintf(term_error,"Term: C_LISTEN '%d %d': %s", sockinfo[k].client, backlog,
       command_result);
     print_err(1);
+    if (i < -1) term_do_exit();
     return -1;
   }
   return 0;
@@ -791,7 +818,7 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
   int s, i, iport, k;
   struct sockaddr_in *name_in;
 
-  if(! use_term) return bind(S,my_addr,addrlen);
+  if(! use_term) return x__bind(S,my_addr,addrlen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -810,7 +837,7 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
   if (k == MAX_CLIENTS) for(i=0;i<pending_count;i++) if(udp_pending[i] == S) {
     if ((k=term_udpsocket(S)) < 0)
       if(use_term < 1) {
-        return bind(S, my_addr, addrlen);
+        return x__bind(S, my_addr, addrlen);
       }else {
         errno = EBADF;
         return -1;
@@ -826,7 +853,7 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
 
     if (name_in->sin_addr.s_addr != inet_addr("127.0.0.254") 
 	&& name_in->sin_addr.s_addr != htonl(term_remoteaddr)) {         
-      j = bind(S, my_addr, addrlen);	/* Try the address requested */
+      j = x__bind(S, my_addr, addrlen);	/* Try the address requested */
       tmperrno = errno;
       errno = tmperrno;
       if (j == -1  && tmperrno != EBADF) return j;
@@ -855,8 +882,8 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
       sprintf(term_error,"Term: C_UBIND failed: %s %d %d",command_result,
         sockinfo[k].client, iport);
       print_err(1);
-    }else if ((i=getsockname(S,&addr,&len)) < 0) {
-      perror("getsockname()");
+    }else if ((i=x__getsockname(S,&addr,&len)) < 0) {
+      x__perror("getsockname()");
     }else if ((i=send_command(s, C_UDPSET, 1, "%d :%s",
       sockinfo[k].client, sockaddr_to_str(&addr,0))) < 0) {
       tmp_errno = ~0;
@@ -877,21 +904,25 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
         || (!remote_connect && !name_in->sin_port
             && name_in->sin_addr.s_addr != htonl(term_remoteaddr)
             && name_in->sin_addr.s_addr != inet_addr("127.0.0.254")))
-      return bind(S,my_addr,addrlen);
+      return x__bind(S,my_addr,addrlen);
 
     if ((iport=ntohs(name_in->sin_port))) {
-      service=getservbyport(iport,"tcp");
+      service=getservbyport(htons(iport),"tcp");
       if (service) 
         if ((service=getservbyname(service->s_name,"term")))
           iport=ntohs(service->s_port);
-      if (!service && !remote_connect && 
-          name_in->sin_addr.s_addr != inet_addr("127.0.0.254") )
-        return bind(S,my_addr,addrlen);
+      if (!service && name_in->sin_addr.s_addr != inet_addr("127.0.0.254")) {
+        if (!remote_connect
+           || ( geteuid() && term_remoteuser && iport < 1024)
+           || ((service=getservbyport(htons(iport),"noterm"))) ){
+          return x__bind(S,my_addr,addrlen);
+        }
+      } 
     }
 
     if (Sock < 0) 
       if ((Sock=socket_connect_server(-1,term_server)) < 0) {
-      return bind(S,my_addr,addrlen);
+      return x__bind(S,my_addr,addrlen);
     }
 
     if ((s = duplicate_connect(S,Sock)) < 0) {
@@ -920,7 +951,7 @@ int term_bind(int S, struct sockaddr *my_addr, int addrlen) {
     }
 
     if(i<0){
-      if(s >= 0) close(s);
+      if(s >= 0) x__close(s);
       return i;
     }
     return 0;
@@ -936,7 +967,7 @@ int term_accept(int pending, struct sockaddr *addr, int *addrlen){
   int j=MAX_CLIENTS,s = -1,i = -1;
   char port[34];
 
-  if(! use_term) return accept(pending,addr,addrlen);
+  if(! use_term) return x__accept(pending,addr,addrlen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -954,11 +985,11 @@ int term_accept(int pending, struct sockaddr *addr, int *addrlen){
       if(pending == sockinfo[j].sock) break;
 
   if (j == MAX_CLIENTS || addr == NULL || addrlen == NULL )
-    return accept(pending, addr, addrlen);
+    return x__accept(pending, addr, addrlen);
 
   if (read(pending,port,10) < 0) {
     tmp_errno = errno;
-    sprintf(term_error,"Term: read port: %s",strerror(j));
+    sprintf(term_error,"Term: read port: %s",x__strerror(j));
     print_err(1);
     return -1;
   }
@@ -1009,7 +1040,7 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
   char host[59],*ihost;
   unsigned long j;
  
-  if(! use_term) return connect(S,addr,addrlen);
+  if(! use_term) return x__connect(S,addr,addrlen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1031,7 +1062,7 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
 
   if(addr_in->sin_family != AF_INET) {
     close_fd(S,2);
-    return connect(S,addr,addrlen);
+    return x__connect(S,addr,addrlen);
   }
 
   j = addr_in->sin_addr.s_addr;
@@ -1042,7 +1073,7 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
   }else {
     if (host_lookup(ihost,sizeof(unsigned long),addr_in->sin_family,0,NULL)) {
       close_fd(S,2);
-      return connect(S,addr,addrlen);
+      return x__connect(S,addr,addrlen);
     } 
 #define UC(a) (unsigned char) ((a) & 0xff)
     sprintf(host,"%u.%u.%u.%u",UC(ihost[0]),UC(ihost[1]),UC(ihost[2]),UC(ihost[3]));
@@ -1051,13 +1082,13 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
 
   if (k == MAX_CLIENTS) for(i=0;i<pending_count;i++) if(udp_pending[i] == S) {
     if ((k=term_udpsocket(S)) < 0) 
-      return connect(S, addr, addrlen);
+      return x__connect(S, addr, addrlen);
     break;
   }
 
   if (k < MAX_CLIENTS && sockinfo[k].udp_addr != NULL) {
     client = sockinfo[k].client;
-    connect(S, sockinfo[k].udp_addr, sizeof(sockinfo[k].udp_addr));
+    x__connect(S, sockinfo[k].udp_addr, sizeof(sockinfo[k].udp_addr));
     if ((s = Sock) < 0) {
       if(term_debug >= 0) {
         tmp_errno = ~0;
@@ -1068,6 +1099,10 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
         client, sockaddr_to_str(addr,0))) < 0) {
       tmp_errno = ~0;
       sprintf(term_error, "Term: C_UDPSET failed: %s",command_result);
+      if (i < -1) {
+        x__close(s);
+        s = -1;
+      }
       print_err(1);
     }
   }else {
@@ -1100,6 +1135,14 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
           print_err(1);
         }
       }else {
+#if 1
+        if (isdigit(ihost[0]) && inet_addr(ihost) == htonl(term_remoteaddr) && ntohs(addr_in->sin_port) == 1080) {
+          addr_in->sin_port = 80;
+          fprintf(stderr,"Redirecting %s:1080 to %s:80\n",ihost,ihost);
+        }else if (ntohs(addr_in->sin_port) == 1080) {
+          fprintf(stderr,"Not Redirecting %s:1080 to %lx:80\n",ihost,term_remoteaddr);
+        }
+#endif
         if ((i=send_command(S,C_PORT,0,"%s:%d",ihost,ntohs(addr_in->sin_port),
             old))< 0) {
           tmp_errno = ~0;
@@ -1117,7 +1160,7 @@ int term_connect(int S, struct sockaddr *addr, int addrlen){
       store_sockinfo(S, client, NULL);
     }
   }
-  return ((i<0) ? connect(S,addr,addrlen) : 0);
+  return ((i<0) ? x__connect(S,addr,addrlen) : 0);
 }
 
 
@@ -1130,7 +1173,7 @@ int term_gethostname(char *name, size_t len) {
   int len2;
   struct hostent *hs=NULL;
 
-  if(! use_term) return gethostname(name, len);
+  if(! use_term) return x__gethostname(name, len);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1143,11 +1186,11 @@ int term_gethostname(char *name, size_t len) {
     print_err(32768);
   }
 
-  if (! remote_connect) return gethostname(name, len);
+  if (! remote_connect) return x__gethostname(name, len);
 
   if ( remote_term_version < 20054) {
     if (!(hs=term_gethostbyname(NULL))) {
-       return gethostname(name, len);
+       return x__gethostname(name, len);
     }
     hs = str_to_hostent(command_result);
     len2 = strlen(hs->h_name)+1;
@@ -1178,7 +1221,7 @@ int term_gethostname(char *name, size_t len) {
 int term_socket( int domain, int type, int protocol) {
   int s,i;
 
-  if(! use_term) return socket(domain, type, protocol);
+  if(! use_term) return x__socket(domain, type, protocol);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1194,7 +1237,7 @@ int term_socket( int domain, int type, int protocol) {
 	/* If the same file descriptor was closed with fclose, then term */
 	/* didn't know about it, so we take care of that now. */
 
-  if ((s = socket(domain,type,protocol)) < 0) return s;
+  if ((s = x__socket(domain,type,protocol)) < 0) return s;
 
   for(i=0;i<MAX_CLIENTS;i++)
     if (s == sockinfo[i].sock) close_fd(s,2);
@@ -1214,8 +1257,13 @@ int term_socket( int domain, int type, int protocol) {
   return s;
 }
 
+#ifndef linux
 int term_sendto(int s, void *msg, int len, unsigned int flags, 
     struct sockaddr *to, int tolen) {
+#else
+int term_sendto(int s, const void *msg, int len, unsigned int flags, 
+    const struct sockaddr *to, int tolen) {
+#endif
 
   int i,k;
   static un_char *buff=NULL;
@@ -1224,7 +1272,8 @@ int term_sendto(int s, void *msg, int len, unsigned int flags,
   char *ihost=NULL;
   struct sockaddr_in *to_addr = (struct sockaddr_in *)to;
 
-  if (! use_term) return sendto(s, msg, len, flags, to, tolen);
+  if (! use_term) return x__sendto(s, (char *)msg, len, flags,
+    (struct sockaddr *)to, tolen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1257,14 +1306,16 @@ int term_sendto(int s, void *msg, int len, unsigned int flags,
     }
     if (k == MAX_CLIENTS || k < 0) {
       if (to && tolen) 
-        return sendto(s, msg, len, flags, to, tolen);
+        return x__sendto(s, (char *)msg, len, flags,
+          (struct sockaddr *)to, tolen);
       else 
-        return send(s, msg, len, flags);
+        return x__send(s, (char *)msg, len, flags);
     }
   }
 
   if (ihost && host_lookup(ihost,sizeof(long unsigned),AF_INET,0,NULL))
-    return sendto(s, msg, len, flags, to, tolen);
+    return x__sendto(s, (char *)msg, len, flags,
+      (struct sockaddr *)to, tolen);
     
 
   if (alloced < (len+HEADER_SIZE)) {	/* malloc it */
@@ -1294,7 +1345,7 @@ int term_sendto(int s, void *msg, int len, unsigned int flags,
   }
   memcpy(buff+HEADER_SIZE,msg,len);
 
-  i = sendto(s, buff, len+HEADER_SIZE, flags, sockinfo[k].udp_addr,
+  i = x__sendto(s, buff, len+HEADER_SIZE, flags, sockinfo[k].udp_addr,
 	sizeof(struct sockaddr));
   
   return (i - HEADER_SIZE);
@@ -1309,13 +1360,19 @@ int term_sendto(int s, void *msg, int len, unsigned int flags,
  * measures here.
  */
 
+#ifndef linux
 int term_rcmd(char **ahost,unsigned short inport, char *locuser,
     char *remuser, char *cmd, int *fd2p){
+#else
+int term_rcmd(char **ahost,unsigned short inport, const char *locuser,
+    const char *remuser, const char *cmd, int *fd2p){
+#endif
   int i = 0,s = -1;
   char *rcommand, *term;
   struct servent *sp;
 
-  if(! use_term) return rcmd(ahost,inport,locuser,remuser,cmd,fd2p);
+  if(! use_term) return x__rcmd(ahost,inport,(char *)locuser,
+    (char *)remuser,(char *)cmd,fd2p);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1333,7 +1390,8 @@ int term_rcmd(char **ahost,unsigned short inport, char *locuser,
   if (! geteuid() && *ahost)
     if (strcmp(*ahost,"remotehost") && strcmp(*ahost,"127.0.0.254"))
       if (use_term < 1 && host_lookup(*ahost,0,AF_INET,0,NULL))
-        return rcmd(ahost,inport,locuser,remuser,cmd,fd2p);
+        return x__rcmd(ahost,inport,(char *)locuser,
+          (char *)remuser,(char *)cmd,fd2p);
 
 /* These values will need to be passed to the rd_exec function */
 
@@ -1356,9 +1414,9 @@ int term_rcmd(char **ahost,unsigned short inport, char *locuser,
     sprintf(term_error,"%s is not understood by term yet.",sp->s_name);
     print_err(1);
     return -1;
-  };
+  }
 
-  s = socket(AF_INET, SOCK_STREAM, 0);
+  s = x__socket(AF_INET, SOCK_STREAM, 0);
   if ((s = socket_connect_server(s,term_server)) <0) {
     tmp_errno = ~0;
     sprintf(term_error, "Term 6: %s", command_result);
@@ -1416,21 +1474,21 @@ int term_rcmd(char **ahost,unsigned short inport, char *locuser,
           term_close(s);
         }else {
           send_command(s, C_DUMB, 1, 0);
-        };
-      };
-    };
-  };
+        }
+      }
+    }
+  }
 
   if (i<0) {
     if (s>=0) {
-      close(s);
+      x__close(s);
       s = -1;
     }
   }else if (fd2p!=NULL) {
     int stat_pipe[2];
     if((pipe(stat_pipe))>=0){ /* open a pipe for passing the connect status */
         *fd2p=stat_pipe[0];
-        close(stat_pipe[1]);
+        x__close(stat_pipe[1]);
     };
   };
   return s;
@@ -1440,8 +1498,7 @@ int term_rcmd(char **ahost,unsigned short inport, char *locuser,
 /* exec, so this should patch most of the security problems. */
 
 int term_fork(void) {
-
-  if(! use_term) return fork();
+  if(! use_term) return x__fork();
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1454,12 +1511,34 @@ int term_fork(void) {
     print_err(32768);
   }
 
-  return fork();
+	/* This is to prevent problems with race conditions... */
+	/* The cost is we will be using an extra client after */
+	/* the next exec().   Fortunately this isn't neccissary */
+	/* with term_vfork(). */
+
+  if (Sock < 0) {
+    return x__fork();
+  }else {
+    int s,pid;
+    s=duplicate_connect(-1,Sock);
+    pid = x__fork();
+    if (pid) {
+      if (s >= 0) close(s);
+      return pid;
+    }else {
+      Sock = s;
+    }
+  }
+  return 0;
 }
 
+#ifndef linux
 int term_chroot(char *path) {
+#else
+int term_chroot(const char *path) {
+#endif
 
-  if(! use_term) return chroot(path);
+  if(! use_term) return x__chroot((char *)path);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1478,12 +1557,12 @@ int term_chroot(char *path) {
     sprintf(term_error, "Term A: %s", command_result);
     print_err(1);
   }
-  return chroot(path);
+  return x__chroot((char *)path);
 }
 
 int term_vfork(void) {
 
-  if(! use_term) return vfork();
+  if(! use_term) return x__vfork();
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1496,7 +1575,7 @@ int term_vfork(void) {
     print_err(32768);
   }
 
-  return vfork();
+  return x__vfork();
 }
 
 /* term_recvfrom()
@@ -1505,15 +1584,20 @@ int term_vfork(void) {
  * to the remote site instead of localhost, and strip header. -danjo
  */
 
-int term_recvfrom( int s, char *buff, int len, int flags,
+#ifndef linux
+int term_recvfrom( int s, char *buff, int len, unsigned int flags,
 		  struct sockaddr *from, int *fromlen)
+#else
+int term_recvfrom( int s, void *buff, int len, unsigned int flags,
+		  struct sockaddr *from, int *fromlen)
+#endif
 {
   static un_char *mybuff = NULL;
   static int alloced;
   int avail, myfromlen, k, h;
   struct sockaddr_in myfrom;
 
-  if(! use_term) return recvfrom(s,buff,len,flags,from,fromlen);
+  if(! use_term) return x__recvfrom(s,(char *)buff,len,flags,from,fromlen);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1528,16 +1612,14 @@ int term_recvfrom( int s, char *buff, int len, int flags,
 
   for(k=0;k<MAX_CLIENTS;k++)
     if (s == sockinfo[k].sock && sockinfo[k].udp_addr != NULL) break;
-  if (k == MAX_CLIENTS) return recvfrom(s,buff,len,flags,from,fromlen);
+  if (k == MAX_CLIENTS) return x__recvfrom(s,(char *)buff,len,flags,from,fromlen);
 
   avail = len + HEADER_SIZE;
 
-#if 1  /* This doesn't seem to work right! */
 #ifdef FIONREAD
   ioctl(s,FIONREAD,&avail);
   if (avail > len + HEADER_SIZE) 
     avail = len + HEADER_SIZE;
-#endif
 #endif
 
   if( alloced < avail) { /* not enough room, let's alloc more */
@@ -1552,13 +1634,14 @@ int term_recvfrom( int s, char *buff, int len, int flags,
 
   h = HEADER_SIZE;
 
-  avail=recvfrom(s,mybuff,avail,flags,(struct sockaddr *)&myfrom,
+  avail=x__recvfrom(s,mybuff,avail,flags,(struct sockaddr *)&myfrom,
       &myfromlen);
   k=errno;
 
   if (avail < h)
     h=0;
-  else if (fromlen != NULL && from != NULL && myfromlen) {
+  else if (fromlen != NULL && from != NULL && sockinfo[k].udp_addr != NULL &&
+     myfromlen) {
     struct sockaddr_in *udp_in;
     udp_in = (struct sockaddr_in *) sockinfo[k].udp_addr;
     if (myfrom.sin_family != udp_in->sin_family
@@ -1568,7 +1651,7 @@ int term_recvfrom( int s, char *buff, int len, int flags,
 
   if (! h) {
     if (avail >= 0)
-      memcpy(buff, mybuff,avail*sizeof(char));
+      memcpy((char *)buff, mybuff,avail*sizeof(char));
   }else {
     avail -= h;
     if (fromlen != NULL && from != NULL) {
@@ -1582,7 +1665,7 @@ int term_recvfrom( int s, char *buff, int len, int flags,
           myfrom.sin_addr.s_addr == INADDR_ANY ) 
         myfrom.sin_addr.s_addr = htonl(term_remoteaddr);
     }
-    memcpy(buff,mybuff+HEADER_SIZE,avail*sizeof(char));
+    memcpy((char *)buff,mybuff+HEADER_SIZE,avail*sizeof(char));
   }     
   if (avail >= 0 && fromlen != NULL && from != NULL) {
     *fromlen = (*fromlen > myfromlen) ? myfromlen : *fromlen;
@@ -1599,9 +1682,13 @@ int term_recvfrom( int s, char *buff, int len, int flags,
  * just call term_recvfrom ignoring the garbage -danjo
  */
 
-int term_recv(int s, char *buf, int len, int flags)
+#ifndef linux
+int term_recv(int s, char *buf, int len, unsigned int flags)
+#else
+int term_recv(int s, void *buf, int len, unsigned int flags)
+#endif
 {
-  if(! use_term) return recv(s,buf,len,flags);
+  if(! use_term) return x__recv(s,(char *)buf,len,flags);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
@@ -1614,7 +1701,7 @@ int term_recv(int s, char *buf, int len, int flags)
     print_err(32768);
   }
 
-  return term_recvfrom(s,buf,len,flags,NULL,NULL);
+  return term_recvfrom(s,(char *)buf,len,flags,NULL,NULL);
 }
 
 /* The following 3 functions all do the same thing, duplicate the 
@@ -1624,9 +1711,9 @@ int term_recv(int s, char *buf, int len, int flags)
 int term_dup(int oldfd) {
   int k, newfd;
 
-  if(! use_term) return dup(oldfd);
+  if(! use_term) return x__dup(oldfd);
 
-  if ((newfd = dup(oldfd)) < 0) return newfd;
+  if ((newfd = x__dup(oldfd)) < 0) return newfd;
 
   if (term_debug & 32768) {
     tmp_errno = errno;
@@ -1651,9 +1738,9 @@ int term_dup(int oldfd) {
 int term_dup2(int oldfd, int newfd) {
   int k, fd;
 
-  if(! use_term) return dup2(oldfd,newfd);
+  if(! use_term) return x__dup2(oldfd,newfd);
 
-  if ((fd = dup2(oldfd,newfd)) < 0) return fd;
+  if ((fd = x__dup2(oldfd,newfd)) < 0) return fd;
 
   if (term_debug & 32768) {
     tmp_errno = errno;
@@ -1675,12 +1762,12 @@ int term_dup2(int oldfd, int newfd) {
 }
 
 
-int term_fcntl(int fd, int cmd, long arg) {
+int term_fcntl(int fd, int cmd, ...) {
   int k, newfd;
 
-  if(! use_term) return fcntl(fd,cmd,arg);
+  if(! use_term) return x__fcntl(fd,cmd,*(long *)(sizeof(int *)+&cmd));
 
-  if ((newfd=fcntl(fd, cmd, arg)) < 0) return newfd;
+  if ((newfd=x__fcntl(fd, cmd, *(long *)(sizeof(int *)+&cmd))) < 0) return newfd;
 
   if (cmd != F_DUPFD) return newfd;
 
@@ -1703,9 +1790,13 @@ int term_fcntl(int fd, int cmd, long arg) {
   return newfd;
 }
 
+#ifndef linux
 int term_send(int s, void *msg, int len, unsigned int flags) {
+#else
+int term_send(int s, const void *msg, int len, unsigned int flags) {
+#endif
 
-  if(! use_term) return send(s,msg,len,flags);
+  if(! use_term) return x__send(s,(char *)msg,len,flags);
 
   if (remote_connect < 0) {
     remote_connect=use_term_command(PUBLIC);
